#include "dxstdafx.h"
#include "Lights.hpp"

const unsigned short N = 10;

Lights::Lights(void) : _vbuffer(0), _ibuffer(0), yaw(0), pitch(0), wireframe(false)
{
}

Lights::~Lights(void)
{
}

HRESULT Lights::onCreateDevice()
{
	VERTEXFORMAT vs[N*N];

	for (int i=0; i<N; ++i)
		for (int j=0; j<N; ++j)
		{
			vs[N*i+j].v = D3DXVECTOR3(1.0f*j/(N-1)-0.5f, 0, 0.5f-1.0f*i/(N-1));
			vs[N*i+j].n = D3DXVECTOR3(0, 1, 0);
		}

		HRESULT hr;
		V( _device->CreateVertexBuffer(sizeof(VERTEXFORMAT) * N*N, 0, FVF,
			D3DPOOL_MANAGED, &_vbuffer, 0) );
		void * vdata;
		V( _vbuffer->Lock(0, sizeof(VERTEXFORMAT) * N*N, &vdata, 0) );
		CopyMemory(vdata, vs, sizeof(VERTEXFORMAT) * N*N);
		_vbuffer->Unlock();

		// Index buffer
		const unsigned int isSize = (N-1)*N*2;
		unsigned short is[isSize];
		unsigned short *isp = is;

		for (unsigned short i=0; i<N-1; ++i)
		{
			*isp++ = N*(i+1);
			*isp++ = N*i;
			for (unsigned short j=1; j<N; ++j)
			{
				*isp++ = N*(i+1)+j;
				*isp++ = N*i+j;
			}
		}


		V( _device->CreateIndexBuffer(sizeof(short) * isSize, 0, D3DFMT_INDEX16,
			D3DPOOL_MANAGED, &_ibuffer, 0) );
		V( _ibuffer->Lock(0, sizeof(short) * isSize, &vdata, 0) );
		CopyMemory(vdata, is, sizeof(short) * isSize);
		_ibuffer->Unlock();

		return D3D_OK;
}


HRESULT Lights::onResetDevice()
{
	D3DXMATRIX pers;

	D3DXMatrixPerspectiveFovLH(&pers, 3.1415f*45/180,
		(float)getWidth()/getHeight(), 0.1f, 10);
	_device->SetTransform(D3DTS_PROJECTION, &pers);
	_device->SetRenderState(D3DRS_LIGHTING, FALSE);

	setWorldViewMatrix();

	return D3D_OK;
}


void Lights::onDestroyDevice()
{
	SAFE_RELEASE(_vbuffer);
	SAFE_RELEASE(_ibuffer);
}


void Lights::onFrameMove( double fTime, float fElapsedTime )
{
}


void Lights::onFrameRender( double fTime, float fElapsedTime )
{
	_device->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
		0xFF6495ED, 1, 0);

	if ( SUCCEEDED(_device->BeginScene()) )
	{
		_device->SetFVF(FVF);
		_device->SetStreamSource(0, _vbuffer, 0, sizeof(VERTEXFORMAT));
		_device->SetIndices(_ibuffer);

		_device->SetTransform(D3DTS_WORLD, &world);
		for (int i=0; i<N-1; ++i)
			_device->DrawIndexedPrimitive(D3DPT_TRIANGLESTRIP, 0, i*N, 2*N, i*((N-1)*2+2), (N-1)*2);


		D3DXMATRIX rot, tra;
		D3DXMatrixRotationX(&rot, -D3DX_PI/2);
		D3DXMatrixTranslation(&tra, 0, 0.5f, 0.5f); 
		rot *= tra * world;
		_device->SetTransform(D3DTS_WORLD, &rot);
		for (int i=0; i<N-1; ++i)
			_device->DrawIndexedPrimitive(D3DPT_TRIANGLESTRIP, 0, i*N, 2*N, i*((N-1)*2+2), (N-1)*2);


		D3DXMatrixRotationZ(&rot, -D3DX_PI/2);
		D3DXMatrixTranslation(&tra, -0.5f, 0.5f, 0); 
		rot *= tra * world;
		_device->SetTransform(D3DTS_WORLD, &rot);
		for (int i=0; i<N-1; ++i)
			_device->DrawIndexedPrimitive(D3DPT_TRIANGLESTRIP, 0, i*N, 2*N, i*((N-1)*2+2), (N-1)*2);


		//_device->DrawPrimitive(D3DPT_POINTLIST, 0, N*N);
		_device->EndScene();
	}
}

void Lights::keyboardProc( UINT nChar, bool bKeyDown, bool bAltDown )
{
	static bool pressed = false;
	if (bKeyDown && !pressed)
	{
		switch (nChar)
		{
		case 'W':
			wireframe = !wireframe;
			if (wireframe)
				_device->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
			else
				_device->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
			break;
		}
	}
	pressed = bKeyDown;
}

void Lights::mouseProc(bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown,
					  bool bSideButton1Down, bool bSideButton2Down, INT nMouseWheelDelta,
					  INT xPos, INT yPos )
{
	static int oldX, oldY;
	static bool pressed = false;

	bool changed = false;

	if (bLeftButtonDown)
	{
		if (!pressed)
			pressed = true;
		else {
			yaw += 2.0f*(oldX-xPos)/this->getWidth();
			pitch += 2.0f*(oldY-yPos)/this->getHeight();
			changed = true;
		}
		oldX = xPos;
		oldY = yPos;
	} else {
		if (pressed)
		{
			pressed = false;
			changed = true;
		}
	}
	if (changed)
		setWorldViewMatrix();
}

void Lights::setWorldViewMatrix()
{
	D3DXMATRIX r1, r2, t;
	D3DXMatrixRotationX(&r1, pitch);
	D3DXMatrixRotationY(&r2, yaw);
	D3DXMatrixTranslation(&t, 0, 0, -0.5f);
	world = r1 * r2 * t;

	D3DXMatrixRotationX(&r1, -3.1415f/4);
	D3DXMatrixTranslation(&t, 0, 0, 3);
	r1 *= t;
	_device->SetTransform(D3DTS_VIEW, &r1);

}